using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Threading;

namespace BibliotecaComum
{
    public class Projeto
    {
        private string _enderecoDir;
        private string _enderecoCfg;
        private ArquivoIni _ini;
        private bool _existia;

        public bool Existia
        {
            get { return _existia; }
        }


        public Projeto(string diretorio)
        {
            _enderecoDir = diretorio;
            if (!Directory.Exists(diretorio))
            {
                Directory.CreateDirectory(diretorio);
            }
            _enderecoCfg = Path.Combine(diretorio, "projeto.hpj");
            _existia = File.Exists(_enderecoCfg);
            _ini = new ArquivoIni(_enderecoCfg);

        }

        public string EnderecoDiretorio
        {
            get
            {
                return _enderecoDir;
            }
        }

        public string ArquivoConfiguracao
        {
            get
            {
                return Path.GetFileName(_enderecoCfg);
            }
        }

        public string Caminho(string arquivo)
        {
            return Path.Combine(_enderecoDir, arquivo);
        }

        /// <summary>
        /// Retorna a extenso sem pontos e caixa baixa.
        /// </summary>
        /// <param name="arquivo">Nome do arquivo</param>
        /// <returns>Extenso sem pontos e caixa baixa</returns>
        public string Extensao(string arquivo)
        {
            return Path.GetExtension(arquivo).ToLower().Replace(".", "");
        }

        public string Nome
        {
            get
            {
                return _ini.Ler("Projeto", "Nome", "Sem nome");
            }
            set
            {
                _ini.Gravar("Projeto", "Nome", value);
            }
        }

        public string Montador
        {
            get
            {
                return _ini.Ler("Projeto", "Montador", "osasm");
            }
            set
            {
                _ini.Gravar("Projeto", "Montador", value);
            }
        }

        public TipoDeProjeto Tipo
        {
            get
            {
                string tipo = _ini.Ler("Projeto", "Tipo", "husis");
                switch (tipo)
                {
                    case "husis":
                        return TipoDeProjeto.ProgramaHUSIS;
                    case "bin":
                        return TipoDeProjeto.BinarioSimples;
                    case "com":
                        return TipoDeProjeto.BinarioDOS;
                    case "nenhum":
                        return TipoDeProjeto.Nenhum;
                    default:
                        throw new NotImplementedException("Tipo de Projeto no suportado");
                }
            }
            set
            {
                string tipo = "";
                switch (value)
                {
                    case TipoDeProjeto.BinarioSimples:
                        tipo = "bin";
                        break;
                    case TipoDeProjeto.ProgramaHUSIS:
                        tipo = "husis";
                        break;
                    case TipoDeProjeto.BinarioDOS:
                        tipo = "com";
                        break;
                    default:
                        break;
                }
                _ini.Gravar("Projeto", "Tipo", tipo);
            }
        }

        public bool TipoHusis
        {
            get
            {
                return Tipo == TipoDeProjeto.ProgramaHUSIS;
            }
            set
            {
                if (value) Tipo = TipoDeProjeto.ProgramaHUSIS;
            }
        }

        public bool TipoBinario
        {
            get
            {
                return Tipo == TipoDeProjeto.BinarioSimples;
            }
            set
            {
                if (value) Tipo = TipoDeProjeto.BinarioSimples;
            }
        }

        public bool TipoBinarioDOS
        {
            get
            {
                return Tipo == TipoDeProjeto.BinarioDOS;
            }
            set
            {
                if (value) Tipo = TipoDeProjeto.BinarioDOS;
            }
        }

        public string RotinaInicial
        {
            get
            {
                return _ini.Ler("Projeto", "RotinaInicial", null);
            }
            set
            {
                _ini.Gravar("Projeto", "RotinaInicial", value);
            }
        }


        public string ArquivoPrincipal
        {
            get
            {
                return _ini.Ler("Projeto", "Arquivo", null);
            }
            set
            {
                _ini.Gravar("Projeto", "Arquivo", value);
            }
        }

        public string ExtensaoAssembly
        {
            get
            {
                return _ini.Ler("Projeto", "ExtensaoAssembly", "s");
            }
            set
            {
                _ini.Gravar("Projeto", "ExtensaoAssembly", value);
            }
        }

        public string ComandoExterno
        {
            get
            {
                return _ini.Ler("Projeto", "Externo", null);
            }
            set
            {
                _ini.Gravar("Projeto", "Externo", value);
            }
        }

        public string[] SubProjetos
        {
            get
            {
                return _ini.Listar("SubProjetos");
            }
        }

        public void AdicionarSubProjeto(string sub)
        {
            _ini.Gravar("SubProjetos", sub, "COMPILAR");
        }

        public void RemoverSubProjeto(string sub)
        {
            _ini.Remover("SubProjetos", sub);
        }

        public string LerSubProjeto(string sub)
        {
            return _ini.Ler("SubProjetos", sub, "");
        }

        public string[] Arquivos
        {
            get
            {
                return _ini.Listar("Arquivos");
            }
        }

        public AcaoArquivo ParaAcaoArquivo(string acao)
        {
            switch (acao.ToLower().Trim())
            {
                case "":
                    return AcaoArquivo.Nenhuma;
                case "asm":
                    return AcaoArquivo.CompilarAssembly;
                case "trad":
                    return AcaoArquivo.CompilarTraducao;
                default:
                    throw new NotImplementedException("Ao de arquivo no implementada");
            }
        }

        public string DeAcaoArquivo(AcaoArquivo acao)
        {
            switch (acao)
            {
                case AcaoArquivo.Nenhuma:
                    return "";
                case AcaoArquivo.CompilarAssembly:
                    return "asm";
                case AcaoArquivo.CompilarTraducao:
                    return "trad";
                default:
                    throw new NotImplementedException("Ao de arquivo no implementada");
            }
        }


        public void Adicionar(string arquivo, AcaoArquivo acao)
        {
            _ini.Gravar("Arquivos", arquivo, DeAcaoArquivo(acao));
        }

        public AcaoArquivo LerAcaoArquivo(string arquivo)
        {
            return ParaAcaoArquivo(_ini.Ler("Arquivos", arquivo, ""));
        }

        public void GravarAcaoArquivo(string arquivo, AcaoArquivo acao)
        {
            _ini.Gravar("Arquivos", arquivo, DeAcaoArquivo(acao));
        }

        public void Remover(string arquivo)
        {
            _ini.Remover("Arquivos", arquivo);
        }


        public void Salvar()
        {
            _ini.SalvarArquivo();
        }

        public bool Compilar(ISaidaCompilacao saida, string enderecoExe, string tradCompilandoProjeto, string tradNaoEncontrado, string asmProg)
        {
            foreach (string sub in SubProjetos)
            {
                if (LerSubProjeto(sub) == "COMPILAR")
                {
                    saida.EscreverNL("+-"+ new string('-', sub.Length) + "-+");
                    saida.EscreverNL("| " + sub + " |");
                    saida.EscreverNL("+-" + new string('-', sub.Length) + "-+");
                    Projeto subProjeto = new Projeto(this.Caminho(sub));
                    saida.EscreverNL(subProjeto.EnderecoDiretorio);
                    if (!subProjeto.Existia)
                    {
                        string erroNaoEncontrado = String.Format(tradNaoEncontrado, Path.GetFileName(sub));
                        saida.Erro(erroNaoEncontrado);
                        return false;
                    }
                    if (!subProjeto.Compilar(saida, enderecoExe, tradCompilandoProjeto, tradNaoEncontrado, asmProg)) return false;
                }
            }
            saida.EscreverNL(tradCompilandoProjeto + this.Nome);
            string conteudo = "";
            string ext = "com";
            string prearq = "";
            string posarq = "";
            foreach (string arq in this.Arquivos)
            {
                if (this.LerAcaoArquivo(arq) == AcaoArquivo.CompilarAssembly)
                {
                    if (arq == this.ArquivoPrincipal)
                    {
                        conteudo = "%include \"" + arq + "\"" + Environment.NewLine + conteudo;
                    }
                    else
                    {
                        conteudo += "%include \"" + arq + "\"" + Environment.NewLine;
                    }
                }
            }
            switch (this.Tipo)
            {
                case TipoDeProjeto.BinarioSimples:
                    ext = "sys";
                    prearq = "prebin.asm";
                    posarq = "posbin.asm";
                    break;
                case TipoDeProjeto.BinarioDOS:
                    ext = "com";
                    prearq = "predos.asm";
                    posarq = "posdos.asm";
                    break;
                case TipoDeProjeto.ProgramaHUSIS:
                    ext = "prg";
                    prearq = "prehusis.asm";
                    posarq = "poshusis.asm";
                    break;
                case TipoDeProjeto.Nenhum:
                    return true;
                default:
                    break;
            }
            string preasm = File.ReadAllText(Path.Combine(Path.GetDirectoryName(enderecoExe), prearq));
            string posasm = File.ReadAllText(Path.Combine(Path.GetDirectoryName(enderecoExe), posarq));
            preasm = preasm.Replace("[ROTINAINICIAL]", this.RotinaInicial);
            posasm = posasm.Replace("[ROTINAINICIAL]", this.RotinaInicial);
            File.WriteAllText(Path.Combine(this.EnderecoDiretorio, "projeto.asm"), preasm + Environment.NewLine + conteudo + Environment.NewLine + posasm);
            if (!File.Exists(asmProg))
            {
                string erroNaoEncontrado = String.Format(tradNaoEncontrado, Path.GetFileName(asmProg));
                saida.Erro(erroNaoEncontrado);
                return false;
            }
            saida.ArquivoExecutavel(Path.Combine(this.EnderecoDiretorio, "projeto." + ext));
            ProcessStartInfo psi = new ProcessStartInfo(asmProg, " -f bin -o projeto." + ext + " projeto.asm");
            saida.EscreverNL("cd \"" + this.EnderecoDiretorio + "\"");
            saida.EscreverNL(psi.FileName + " " + psi.Arguments);
            psi.WorkingDirectory = this.EnderecoDiretorio;
            psi.RedirectStandardOutput = true;
            psi.RedirectStandardError = true;
            psi.UseShellExecute = false;
            psi.CreateNoWindow = true;
            saida.AtualizarInterface();
            Process proc = Process.Start(psi);
            char[] temp = new char[2048];
            int tam;
            bool ret = true;
            while (true)
            {
                while ((tam = proc.StandardOutput.Read(temp, 0, temp.Length)) > 0)
                {
                    saida.Escrever(new string(temp, 0, tam));
                    Thread.Sleep(100);
                    saida.AtualizarInterface();
                }
                while ((tam = proc.StandardError.Read(temp, 0, temp.Length)) > 0)
                {
                    ret = false;
                    saida.Retorno(new string(temp, 0, tam));
                    Thread.Sleep(100);
                    saida.AtualizarInterface();
                }
                Thread.Sleep(100);
                saida.AtualizarInterface();
                if (proc.HasExited) break;
            }
            return ret;
        }
    }

    public enum TipoDeProjeto
    {
        BinarioSimples,
        BinarioDOS,
        ProgramaHUSIS,
        Nenhum
    }

    public enum AcaoArquivo
    {
        Nenhuma,
        CompilarAssembly,
        CompilarTraducao
    }
}
